"""
Alanube API Client

This module provides the Alanube API Client to interact with the Alanube API,
including country-specific implementations. Currently, it includes the
implementation for the Dominican Republic (DGII) under the 'do' directory.
Future implementations will include support for other countries such as Panama,
Costa Rica, and Bolivia.

Country-specific directories:
- `do`: Implementation for the Dominican Republic (DGII)
- `pa`: Implementation for Panama (future)
- `cr`: Implementation for Costa Rica (future)
- `bo`: Implementation for Bolivia (future)
"""

__version__ = "0.1.0"
__author__ = "Wilmer Martínez"
__email__ = "info@wilmermartinez.dev"
__license__ = "MIT"